#? Print in different colors ----------------------------------------------
#! Red 
def red(*args, end="\n"):
    print("\033[91m {}\033[00m".format(''.join(map(str, args))), end=end)

#! Green
def green(*args, end="\n"):
    print("\033[92m {}\033[00m".format(''.join(map(str, args))), end=end)

#! Yellow
def yellow(*args, end="\n"):
    print("\033[93m {}\033[00m".format(''.join(map(str, args))), end=end)

#! Blue
def blue(*args, end="\n"):
    print("\033[94m {}\033[00m".format(''.join(map(str, args))), end=end)

#! Purple
def purple(*args, end="\n"):
    print("\033[95m {}\033[00m".format(''.join(map(str, args))), end=end)

#! Cyan
def cyan(*args, end="\n"):
    print("\033[96m {}\033[00m".format(''.join(map(str, args))), end=end)

#! Light Gray
def lgray(*args, end="\n"):
    print("\033[97m {}\033[00m".format(''.join(map(str, args))), end=end)

#! Dark Gray
def dgray(*args, end="\n"):
    print("\033[90m {}\033[00m".format(''.join(map(str, args))), end=end)

#? Print in different styles ---------------------------------------------
#! Bold
def bold(*args, end="\n"):
    print("\033[1m {}\033[0m".format(' '.join(map(str, args))), end=end)

#! Underline
def underline(*args, end="\n"):
    print("\033[4m {}\033[0m".format(' '.join(map(str, args))), end=end)

#! Negative
def negative(*args, end="\n"):
    print("\033[3m {}\033[0m".format(' '.join(map(str, args))), end=end)

#? Print in different background colors ----------------------------------

#! Red Background
def red_bg(*args, end="\n"):
    print("\033[41m {}\033[00m".format(' '.join(map(str, args))), end=end)

#! Green Background
def green_bg(*args, end="\n"):
    print("\033[42m {}\033[00m".format(' '.join(map(str, args))), end=end)

#! Yellow Background
def yellow_bg(*args, end="\n"):
    print("\033[43m {}\033[00m".format(' '.join(map(str, args))), end=end)

#! Blue Background
def blue_bg(*args, end="\n"):
    print("\033[44m {}\033[00m".format(' '.join(map(str, args))), end=end)

#! Purple Background
def purple_bg(*args, end="\n"):
    print("\033[45m {}\033[00m".format(' '.join(map(str, args))), end=end)

#! Cyan Background
def cyan_bg(*args, end="\n"):
    print("\033[46m {}\033[00m".format(' '.join(map(str, args))), end=end)

#! Light Gray Background
def lgray_bg(*args, end="\n"):
    print("\033[47m {}\033[00m".format(' '.join(map(str, args))), end=end)

#! Dark Gray Background
def dgray_bg(*args, end="\n"):
    print("\033[100m {}\033[00m".format(' '.join(map(str, args))), end=end)

#? Other -----------------------------------------------------------------

#! Easy New Line
def n(n=1):
    for i in range(n):
        print()




from datetime import datetime
import inspect
import ultraprint.common as p

class logger:
    
    def __init__(self, name, filename=None, include_extra_info=False, write_to_file=True, log_level='INFO'):
        self.name = name
        self.include_extra_info = include_extra_info
        self.write_to_file = True

        if filename:
            self.filename = filename
        else:
            self.filename = f'{name}.log'

        self.log_levels = {
            'DEBUG': 0,
            'INFO': 1,
            'SUCCESS': 2,
            'WARNING': 3,
            'ERROR': 4,
            'CRITICAL': 5
        }

        self.current_log_level = self.log_levels[log_level]

    def set_log_level(self, log_level = 'INFO'):
        self.current_log_level = self.log_levels[log_level]
    
    def set_write_to_file(self, write_to_file):
        self.write_to_file = write_to_file

    def _get_extra_info(self):
        frame = inspect.currentframe().f_back.f_back
        info = inspect.getframeinfo(frame)
        return f'{info.filename}:{info.function}:{info.lineno}'

    def _log(self, level, msg, color_func):
        current_time = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        extra_info = self._get_extra_info() if self.include_extra_info else ''
        if self.include_extra_info:
            extra_info = self._get_extra_info()
            formatted_msg = f'[{current_time}] [{level}] [{self.name}] {msg} [{extra_info}]'
        else:
            formatted_msg = f'[{current_time}] [{level}] [{self.name}] {msg}'
        
        if self.log_levels[level] >= self.current_log_level:
            color_func(formatted_msg)
        
        if self.write_to_file:
            with open(self.filename, 'a') as f:
                f.write(f'{formatted_msg}\n')

    def info(self, msg):
        self._log('INFO', msg, p.cyan)

    def error(self, msg):
        self._log('ERROR', msg, p.red)

    def warning(self, msg):
        self._log('WARNING', msg, p.yellow)

    def success(self, msg):
        self._log('SUCCESS', msg, p.green)

    def debug(self, msg):
        self._log('DEBUG', msg, p.dgray)

    def critical(self, msg):
        self._log('CRITICAL', msg, p.red_bg)


from setuptools import setup, find_packages

setup(
    name='ultraprint',
    version='3.0.0',
    license="MIT License with attribution requirement",
    author="Ranit Bhowmick",
    author_email='bhowmickranitking@duck.com',
    description='''Ultraprint is a versatile Python library that enhances your command-line output with colorful text, styled formatting, and easy-to-use logging utilities. Perfect for creating visually appealing console applications.''',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    url='https://github.com/Kawai-Senpai/UltraPrint',
    download_url='https://github.com/Kawai-Senpai/UltraPrint',
    keywords=["Printing", "Console", "Formatting", "Logging", "Colored Output", "Styled Text"],
    install_requires=[],
)
